/*
	json-match
	author:zinkey 
*/
(function (name, factory, context) {  
    
    'use strict';

    if (typeof module != 'undefined' && module.exports) {  
        module.exports = factory();  
    }   
    else {  
        context[name] = factory();  
    }  

})("jsonmatch",function(){

	var isArray = [].constructor.isArray || function(src) {
		return ({}).toString.call(src) === '[object Array]';
	};

	function isString(src){
		return typeof src === "string";
	}

	function isObject(src) {
		return typeof src === "object"&&src!==null;
	}

	function each(src, fn) {

		if (!isObject(src)) {
			return;
		}

		if (isArray(src)) {
			if (src.forEach) {
				src.forEach(fn);
			} else {
				for (var i = 0; i < src.length; i++) {
					fn(src[i], i);
				}
			}
		} else {
			for (var i in src) {
				if (src.hasOwnProperty(i)) {
					fn(src[i], i);
				}
			}
		}

	}

	function getLength(src) {
		if (Object.keys) {
			return Object.keys(src).length
		}
		var len = 0;
		each(src, function(item) {
			len++;
		});
		return len;
	};

	var objectKeys = Object.keys || function(obj) {
		var res = [];
		for (var key in obj) res.push(key)
		return res;
	};


	function traverse(obj, fn) {

		var key = obj.key;
		var src = obj.src;
		var parent = obj.parent;

		if (isObject(src)) {

			var len = getLength(src);

			fn({
				type: "\1",
				key: key,
				src: len,
				parent: parent
			});

			each(src, function(item, key) {
				traverse({
					key: key,
					src: item,
					parent: src
				}, fn);
			});

			fn({
				type: "\2",
				key: key,
				src: len,
				parent: parent
			});

		} else {

			fn({
				type: "\3",
				key: key,
				src: src,
				parent: parent
			});

		}

	}

	function jsonarray(json) {
		var array = [];
		var list = [];
		traverse({
			src: json,
		}, function(obj) {
			list.push(obj.parent);
			array.push(
				obj.type+"\0"+
				obj.key+"\0" + 
				isString(obj.src)?"\4"+obj.src+"\4":obj.src
			);
		});
		array.shift();
		array.pop();
		return {
			list:list,
			array:array
		};
	}

	function equal(a, b) {
		return a === b;
	}

	function occInit(array, pattern) {

		var occ = {};

		each(array,function(item){
			occ[item] = -1;
		});

		each(pattern,function(item,key){
			occ[item] = key;
		});

		return occ;

	}

	function matchesAt(array,pattern,index,eq) {
		for (var i = pattern.length - 1; i >= 0; i--) {
			if (!eq(pattern[i], array[index + i])) {
				return false;
			}
		}
		return true;
	}

	function matchArray(obj,objp,eq) {
		
		var list = obj.list;
		var array = obj.array;
		var pattern = objp.array;

		var occ = occInit(array,pattern);

		var index = 0;
		var result = [];
		var len = array.length - pattern.length;

		while (index <= len) {
			if (matchesAt(array,pattern,index,eq)) {
				result.push(list[index]);
				index += 1;
			} else {
				var last = index + pattern.length;
				if (last < array.length) {
					index += pattern.length - occ[array[last]];
				} else {
					break;
				}
			}
		}

		return result;

	}

	return function(json,pattern,eq){
		return matchArray(jsonarray(json),jsonarray(pattern),eq||equal);
	};

},this);